<?php
/**
 *
 * Auther: heyu
 * DateTime: 2019-09-04 14:09
 *
 */

namespace kwy\util\http;


class CurlHttp
{

    private $url='';

    private $lastRequestTime=0;

    private $contentType='';

    private $isPost=false;

    private $proxy='';

    private $lastRequestLog='';

    private $timeOut=5;

    private $responseType='string';

    private $cookies='';

    /**
     * 超时时间
     * @param int $timeOut 秒
     * @return self
     */
    public function setTimeOut($timeOut)
    {
        $this->timeOut = $timeOut;
        return $this;
    }


    /**
     * @param string $url
     * @return bool|mixed|string
     * @throws \Exception
     */
    public function get($url=''){
        if (!empty($url)){
            $this->setUrl($url);
        }
        $this->isPost=false;
        return $this->request();
    }

    /**
     * @param mixed $data
     * @return string
     * @throws \Exception
     */
    public function post($data){
        $this->isPost=true;
        return $this->request($data);
    }

    /**
     * @return string
     */
    public function getResponseType(): string
    {
        return $this->responseType;
    }

    /**
     * @param string $responseType
     */
    public function setResponseType(string $responseType)
    {
        $this->responseType = $responseType;
    }



    /**
     * @return string
     */
    public function getLastRequestLog()
    {
        return $this->lastRequestLog;
    }

    /**
     * @param string $url
     * @return self
     */
    public function setUrl($url)
    {
        $this->url = $url;
        return $this;
    }

    /**
     * @return int
     */
    public function getLastRequestTime()
    {
        return $this->lastRequestTime;
    }

    /**
     * @return string
     */
    public function getContentType()
    {
        return $this->contentType;
    }

    /**
     * @param string $contentType
     * @return self
     */
    public function setContentType($contentType)
    {
        $this->contentType = $contentType;
        return $this;
    }

    /**
     * @return string
     */
    public function getCookies(): string
    {
        return $this->cookies;
    }

    /**
     * @param string $cookies
     */
    public function setCookies(string $cookies)
    {
        $this->cookies = $cookies;
    }

    /**
     * @return bool
     */
    public function isPost()
    {
        return $this->isPost;
    }

    /**
     * @return string
     */
    public function getProxy()
    {
        return $this->proxy;
    }

    /**
     * @param string $proxy "127.0.0.1:8888"
     * @return self
     */
    public function setProxy($proxy)
    {
        $this->proxy = $proxy;
        return $this;
    }

    public function __construct($url='')
    {
        $this->url=$url;
    }

    /**
     * @param mixed $data default=null
     * @return bool|mixed|string
     * @throws \Exception
     */
    private function request($data=null){
        $api_req_time_b=microtime(true);
        $ch=curl_init();
        $this->lastRequestLog='['.date('Y-m-d H:i:s').'] '.
            (($this->isPost)?'POST':'GET').
            ':'.$this->url.'  [body] '.(is_string($data)?$data:json_encode($data));
        if (empty($this->url)){
            throw new \Exception('请求url不能为空');
        }
        curl_setopt($ch, CURLOPT_URL, $this->url);//url
        curl_setopt($ch, CURLOPT_POST, $this->isPost);//post
        if (!empty($this->contentType)){
            curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type:'.$this->contentType.'; charset=utf-8']);
        }
        //curl_setopt($ch, CURLOPT_HTTPHEADER, ["Content-Type:multipart/form-data; charset=utf-8"]);
        //curl_setopt($ch, CURLOPT_HTTPHEADER, ["Content-Type:text/xml; charset=utf-8"]);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);//返回结果curl
        curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($ch, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4 );//强制ipv4解析
        if (!empty($data))
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);      //忽略校验证书
        //curl_setopt($ch, CURLINFO_HEADER_OUT, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->timeOut);

        ##cookie
        if (!empty($this->cookies)){
            curl_setopt($ch,CURLOPT_COOKIE,$this->cookies);
        }

        #代理访问
        if (!empty($this->proxy)){
            $addr=$this->proxy;
            $port='8888';
            if (strpos($this->proxy,':')!==false){
                $exp=explode(':',$this->proxy);
                $addr=$exp[0];
                $port=$exp[1];
            }
            curl_setopt($ch, CURLOPT_PROXY, $addr); //代理服务器地址
            curl_setopt($ch, CURLOPT_PROXYPORT,$port); //代理服务器端口
        }
        $ret = curl_exec($ch);
        if ($ret===false){
            $ch_no=curl_errno($ch);
            $ch_msg=curl_error($ch);
            curl_close($ch);
            $errMsg='网络请求异常:curl('.$ch_no.'):'.$ch_msg;
            $this->lastRequestTime=microtime(true)-$api_req_time_b;
            $this->lastRequestLog.=' [curl time]'.$this->lastRequestTime.' [result] [curl error]'.$errMsg;
            throw new \Exception($errMsg);
        }
        curl_close($ch);
        $this->lastRequestTime=microtime(true)-$api_req_time_b;
        $this->lastRequestLog.=' [curl time]'.$this->lastRequestTime.' [result] '.$ret;

        if ($this->responseType=='array'){
            return json_decode($ret,true);
        }else{
            return $ret;
        }

    }
}